'--------------------------------------------------------------------
'                  Schaltmodul.BAS
'  Schaltmodul fuer Modelleisenbahnen mit ATiny26
'
'  Autor:   H. Wendt
'
'  Version: 1.2.0
'
'  History:

'  V1.2.0   Funktion fuer Fallerschranke hinzugefgt

'  V1.1.0   Versuche zur Speicheroptimierung

'  V1.0.0   first release
'--------------------------------------------------------------------
$regfile = "at26def.dat"
$crystal = 1000000
Declare Sub Reed_amplifier_ch12
Declare Sub Reed_amplifier_ch34
Declare Sub Weichenfolgeschaltung
Declare Sub Bahnuebergang1
Declare Sub Bahnuebergang2x

'Config AD converter
Config Adc = Single , Prescaler = Auto , Reference = Off
'Now give power to the chip
Start Adc

'With STOP ADC, you can remove the power from the chip
'Stop Adc

'Config Timer for 100ms Interruptserviceroutine
Config Timer0 = Timer , Prescale = 1024
On Ovf0 Timer0_isr
Enable Timer0
Enable Interrupts


Dim Rel1an As Byte , Rel1ab As Word     'counter for relay 1
Dim Rel2an As Word , Rel2ab As Byte     'counter for relay 2
Dim Rel3an As Byte , Rel3ab As Byte     'counter for relay 3
Dim Rel4an As Byte , Rel4ab As Byte     'counter for relay 4


Dim Adc01 As Byte                       'potentiometer for relay 1
Dim Adc02 As Byte                       'potentiometer for relay 2
Dim Adc03 As Byte                       'potentiometer for relay 3
Dim Adc04 As Byte                       'potentiometer for relay 4
Dim Wmode As Byte                       'working mode
Dim Oldmode As Byte                     'old working mode
Dim Chx As Byte                         'last channel

set


'--------------------------------------------------------------------
' Main program
'--------------------------------------------------------------------
Ddra = &B00000000
Ddrb = &B01111000
Portb = &B00000111
Oldmode = 255

Do
 'check potentiometer
 '1. Modul
 'Adc01 = Getadc(6)
 'Adc02 = Getadc(5)
 'Adc03 = Getadc(4)
 'Adc04 = Getadc(3)

 '2. Modul
 Adc01 = Getadc(3)
 Adc02 = Getadc(4)
 Adc03 = Getadc(5)
 Adc04 = Getadc(6)


 'check jumper for working mode
 Wmode = Pinb And &H07

 'Bei nderung der Jumpereinstellung Relais entsprechend schalten
 If Oldmode <> Wmode Then
   'Diesen Zweig nur 1x nach nderung Jumper ausfhren
   Oldmode = Wmode

   'Alle Relais ausschalten!
   Portb = &B00000111

   'Bei Steuerung Bahnbergang Relais 2 einschalten
   If Wmode = 5 Or Wmode = 6 Then Set Portb.5

 End If


 'Entsprechend eingestelltem Modus verzweigen
 If Pinb.1 = 0 Then

            Call Bahnuebergang1         'Zweigleisiger Bahnbergang
            Call Bahnuebergang2x


 Elseif Pinb.0 = 0 Then
            Call Bahnuebergang1         'Eingleisiger Bahnuebergang
            Call Reed_amplifier_ch34

 Elseif Pinb.2 = 0 Then
            Call Weichenfolgeschaltung  'Weichenfolgeschaltung

 Else
            Call Reed_amplifier_ch12    'Reed amplifier
            Call Reed_amplifier_ch34

 End If

 Loop
End


'--------------------------------------------------------------------
' Amplifier for reed contacts
'--------------------------------------------------------------------
Sub Reed_amplifier_ch12
 Debounce Pina.0 , 1 , Reed_relay1 , Sub
 Debounce Pina.1 , 1 , Reed_relay2 , Sub
End Sub

Sub Reed_amplifier_ch34
 Debounce Pina.2 , 1 , Reed_relay3 , Sub
 Debounce Pina.3 , 1 , Reed_relay4 , Sub
End Sub


Reed_relay1:
 Set Portb.6
 Shift Adc01 , Right , 3
 Rel1ab = 5 + Adc01
Return

Reed_relay2:
 Set Portb.5
 Shift Adc02 , Right , 3
 Rel2ab = 5 + Adc02
Return

Reed_relay3:
 Set Portb.4
 Shift Adc03 , Right , 3
 Rel3ab = 5 + Adc03
Return

Reed_relay4:
 Set Portb.3
 Shift Adc04 , Right , 3
 Rel4ab = 5 + Adc04
Return


'--------------------------------------------------------------------
' eingleisiger Bahnbergang
'--------------------------------------------------------------------
Sub Bahnuebergang1
 'Shift Adc01 , Right , 2                'Delay time 1 - 26,5s
 'Shift Adc02 , Right , 2                'Delay time 1 - 26,5s
 Debounce Pina.0 , 1 , Bahnx1 , Sub
 Debounce Pina.1 , 1 , Bahnx2 , Sub
End Sub


Bahnx1:
  If Rel2an > 0 And Chx <> 1 Then
   Rel2an = Adc01 + 10
   If Pinb.2 = 1 Then Rel1ab = Rel2an   'Keine Fallerschranke
  Else
   Chx = 1
   Goto Bahnx3
  End If
Return


Bahnx2:
  If Rel2an > 0 And Chx <> 2 Then
   Rel2an = Adc02 + 10
   If Pinb.2 = 1 Then Rel1ab = Rel2an   'Keine Fallerschranke
  Else
   Chx = 2
   Goto Bahnx3
  End If
Return


Bahnx3:
   Set Portb.6
   Reset Portb.5
   Rel2an = 320
   If Pinb.2 = 1 Then
   'Keine Fallerschranke
    Rel1ab = 320
   Else
   'Fallerschranke
    Rel1ab = 15
    Rel2ab = 15
   End If
Return


'--------------------------------------------------------------------
' zweigleisiger Bahnbergang
'--------------------------------------------------------------------
Sub Bahnuebergang2x


 Shift Adc03 , Right , 4                'interval time 0,5 - 2s
 Shift Adc04 , Right , 4                'interval time 0,2 - 1.7s

 'Interval Relais3
 If Rel3an = 0 And Pinb.4 = 0 And Chx > 0 Then
  Rel3an = Adc03 + 5
 Elseif Rel3ab = 0 And Pinb.4 > 0 Then
  'Rel3ab = Adc03 + 2
  Rel3ab = 5
 End If

 'Interval Relais4
 If Rel4an = 0 And Pinb.3 = 0 And Chx > 0 Then
  Rel4an = Adc04 + 2
 Elseif Rel4ab = 0 And Pinb.3 > 0 Then
  Rel4ab = Adc04 + 2
 End If

 Debounce Pina.2 , 1 , Bahnx1 , Sub
 Debounce Pina.3 , 1 , Bahnx2 , Sub
End Sub


'--------------------------------------------------------------------
' Weichenfolgeschaltung
'--------------------------------------------------------------------
Sub Weichenfolgeschaltung

 Shift Adc01 , Right , 4                'Delay time 1 - 2,5s
 Shift Adc03 , Right , 4                'ON for 1. Weiche 0.5 - 2s
 Shift Adc04 , Right , 4                'ON for 2. Weiche 0.5 - 2s
 Adc01 = Adc01 + 10


 If Rel4an = 0 And Rel4ab = 0 Then Debounce Pina.0 , 1 , Weiche1_3 , Sub
 If Rel3an = 0 And Rel3ab = 0 Then Debounce Pina.1 , 1 , Weiche2_4 , Sub
End Sub

Weiche1_3:
  Set Portb.6
  Rel1ab = Adc03 + 5
  Rel3an = Adc01 + Rel1ab
  Rel3ab = Adc04 + 5
Return

Weiche2_4:
  Set Portb.5
  Rel2ab = Adc03 + 5
  Rel4an = Adc01 + Rel2ab
  Rel4ab = Adc04 + 5
Return



'--------------------------------------------------------------------
' ISR for timer0 overflow
'--------------------------------------------------------------------
Timer0_isr:
'interrupt service routine
'called all 100 ms
 Dim A1 As Byte
 Stop Timer0

 'restart timer0
 Tcnt0 = 158                            '255 - 157 = 98, 98 * 1024/1Mhz = 0,1 s
 Start Timer0

 'Bearbeitung Relais 1
 'Einschalten
 If Rel1an > 0 Then
  Decr Rel1an
 Else
  'Ausschalten
  If Rel1ab > 0 Then Decr Rel1ab
  If Rel1ab = 1 Then
   Reset Portb.6
   'Chx = 0                              'Merker fuer letzen Kanal lschen
  End If
 End If
 If Rel1an = 1 Then Set Portb.6

 'Bearbeitung Relais 2
 'Einschalten
 If Rel2an > 0 Then
  Decr Rel2an
 Else
  'Ausschalten
  If Rel2ab > 0 Then Decr Rel2ab
  If Rel2ab = 1 Then Reset Portb.5
 End If
 If Rel2an = 1 Then
  Set Portb.5
  Chx = 0                              'Merker fuer letzen Kanal lschen
 End If

 'Bearbeitung Relais 3
 'Einschalten
 If Rel3an > 0 Then
  Decr Rel3an
 Else
  'Ausschalten
  If Rel3ab > 0 Then Decr Rel3ab
  If Rel3ab = 1 Then Reset Portb.4
 End If
 If Rel3an = 1 Then Set Portb.4

 'Bearbeitung Relais 4
 'Einschalten
 If Rel4an > 0 Then
  Decr Rel4an
 Else
  'Ausschalten
  If Rel4ab > 0 Then Decr Rel4ab
  If Rel4ab = 1 Then Reset Portb.3
 End If
 If Rel4an = 1 Then Set Portb.3
Return